#!/bin/sh
export A=x86_64-x32
export K=/src/KEEP
export S=/src
export R=/
export C=/src/tarballs
export LOGPATH=/src/logs
export MAKE_THREADS=8
[ -z "$CC" ] && export CC=gcc
export HOSTCC="$CC"
[ -z "$CXX" ] && export CXX=g++
export HOSTCXX="$CXX"

export BUTCH_BUILD_TEMPLATE="$K"/butch_build_template.txt
export BUTCH_DOWNLOAD_TEMPLATE="$K"/butch_download_template.txt
export STAGE=1
skip_build=true
filesize=3368432
sha512=bbe3cb82c35a78331067ce098396acabf83599d7061e3b8ddc063411327dca8ba0ba4e1b258a25d6a37a124342f381dd77c7e70c04adb508f6ddc18e437cd73f
tarball=util-linux-$(cat "$K"/util-linux.version).tar.xz
BUTCH_PACKAGE_NAME=util-linux-tarball
BUTCH_MIRROR_URLS="http://www.kernel.org/pub/linux/utils/util-linux/v2.23/$tarball
http://ftp.free.fr/mirrors/ftp.kernel.org/linux/utils/util-linux/v2.23/$tarball"
get_file_name() {
printf "%s" "$1" | awk '{n=split($0,a,"/");print a[n];}'
}
get_file_size() {
wc -c "$1" | cut -d " " -f 1
}
check_sha512() {
[ $(sha512sum "$2"|cut -d " " -f 1) = "$1" ] && return 0
return 1
}
fail() {
printf "ERROR: %s\n" "$1" >&2
}

butch_cache_dir="$C"

if [ -z "$tarball" ] ; then
for i in $BUTCH_MIRROR_URLS ; do
tarball=$(get_file_name "$i")
break
done
fi
check_tarball_size() {
if [ -n "$filesize" ] && [ "$filesize" != $(get_file_size "$1") ] ; then
fail "FILESIZE MISMATCH"
return 22
fi
return 0
}
check_tarball() {
[ -e "$1" ] || return 1
if ! check_tarball_size "$1" ; then
return 22
fi
if [ -n "$sha512" ] && ! check_sha512 "$sha512" "$1" ; then
fail "SHA512 CHECKSUM MISMATCH"
return 11
fi
return 0
}

# check if tarball already exists
if [ -e "$C"/"$tarball" ] ; then
check_tarball_size "$C"/"$tarball" && {
echo "tarball existing, passed size check. exiting."
exit 0
}
fail "tarball exists but size differs from recipe"
exit 99
fi

# try to download from SCP_MIRRORS list (newline-delimited)
if test x != x"$SCP_MIRRORS" ; then
(
n=1
IFS='
'
for m in $SCP_MIRRORS ; do
IFS=' '
cmd=$(printf "$m" "$tarball" "$C"/"$tarball")
$cmd </dev/null && check_tarball "$C"/"$tarball" && {
echo "successfully downloaded from SCP_MIRROR $n"
exit 0
}
n=$(($n + 1))
IFS='
'
done
exit 1
)
test $? = 0 && test -e "$C"/"$tarball" && exit 0
fi

# try regular ftp/http/https download using the package's mirrors section.
retries=5
tmp=/tmp/butchdl.$$
rc=0
append() {
eval "$1=\"\${$1} \$2\""
eval "$1=\${$1# }"
}

download() {
local url="$1"
local out="$2"
if [ "$USE_CURL" = 1 ] ; then
curl -c /dev/null -C - --retry $retries -k -L "$url" -o "$out"
else
WGET=wget
is_bb=false
$WGET --help 2>&1 | grep BusyBox > /dev/null && is_bb=true
if ! $is_bb && ! $WGET --help >/dev/null 2>&1 ; then
# appears it is GNU wget and it is broken, eventually
# through a libressl soname bump. fall back to busybox.
WGET="busybox wget"
is_bb=true
fi
if ! $is_bb ; then
append wgetflags "--tries=$retries"
append wgetflags --no-check-certificate
fi
$WGET $wgetflags -O "$out" -c "$url"
fi
}

cleanup() {
[ -e "$tmp" ] && { cat $tmp ; rm $tmp ; }
}

FALLBACK_MIRROR=http://tar.sabo.xyz
FALLBACK=$FALLBACK_MIRROR/$BUTCH_PACKAGE_NAME/$tarball
echo "in case the download process was killed, see $tmp for wget/curl output"

for url in $BUTCH_MIRROR_URLS $FALLBACK ; do
if ! download "$url" "$butch_cache_dir/$tarball".wip >$tmp 2>&1 ; then
rc=1
grep 404 < $tmp > /dev/null && { rc=44 ; fail "HTTP 404 FILE NOT FOUND" ; }
[ "$rc" = 1 ] && fail "UNKNOWN DL ERROR, SEE LOG"
cleanup
else
mv "$butch_cache_dir"/"$tarball".wip \
"$butch_cache_dir"/"$tarball"
check_tarball "$butch_cache_dir"/"$tarball"
rc=$?
break
fi
done
cleanup
exit $rc
